#include	<stdio.h>

#include	"tinyda.h"


int		main( int argc, char *argv[] )
{
	int		i, ret;
	int		*val, idx;
	DAStat	stat;
	hDASet	set;
	hDADic	dic;
	const char	*s, *start;

	// キー集合を作成
	if ( ( ret = DASet_Open( &set ) ) != DA_OK )
		fprintf( stderr, "DASet_Open(): %d\n", ret );

	// キー集合をファイルで入力
	for ( i = 1; i < argc; i++ )
	{
		// タブ，改行による区切り
		ret = DASet_Load( set, argv[i], "\t\r\n", sizeof( int ) );
		if ( ret != DA_OK )
			fprintf( stderr, "DADic_Load(): %s: %d\n", argv[i], ret );
	}

	// キー集合にキーを追加
	if ( ( ret = DASet_Insert( set, "Tes", sizeof( int ) ) ) != DA_OK )
		fprintf( stderr, "DADic_Insert(): %d\n", ret );
	if ( ( ret = DASet_Insert( set, "Test", sizeof( int ) ) ) != DA_OK )
		fprintf( stderr, "DADic_Insert(): %d\n", ret );
	if ( ( ret = DASet_Insert( set, "Test-Key", sizeof( int ) ) ) != DA_OK )
		fprintf( stderr, "DADic_Insert(): %d\n", ret );

	// 辞書を構築
	if ( ( ret = DADic_Build( &dic, set, sizeof( int ) ) ) != DA_OK )
		fprintf( stderr, "DADic_Build(): %d\n", ret );

	// キー集合を破棄
	if ( ( ret = DASet_Close( set ) ) != DA_OK )
		fprintf( stderr, "DASet_Close(): %d\n", ret );

	// 辞書の情報を出力
	if ( ( ret = DADic_Status( dic, &stat ) ) != DA_OK )
		fprintf( stderr, "DADic_Stat(): %d\n", ret );
	printf( "Size: 4 * %d + %d = %d\n", stat.nbc, stat.ntail, stat.size );
	printf( "Key: %d\n", stat.nkey );
	printf( "Boundary: %d\n", stat.boundary );

	// キーの検索とレコードの設定
	if ( ( val = DADic_Search( dic, "Test" ) ) == NULL )
		fprintf( stderr, "DADic_Search(): Not found\n" );
	else
		*val = 100;

	// 辞書を出力
	if ( ( ret = DADic_Save( dic, "test.dic" ) ) != DA_OK )
		fprintf( stderr, "DADic_Save(): %d\n", ret );

	// 辞書を破棄
	if ( ( ret = DADic_Close( dic ) ) != DA_OK )
		fprintf( stderr, "DADic_Close(): %d\n", ret );

	// 辞書を入力
	if ( ( ret = DADic_Open( &dic, "test.dic" ) ) != DA_OK )
		fprintf( stderr, "DADic_Open(): %d\n", ret );

	// 前方一致検索
	idx = 0;
	start = s = "Test-Keys";
	while ( ( val = DADic_Prefix( dic, &s, &idx ) ) != NULL )
		printf( "DADic_Prefix(): %.*s: %d\n", s - start, start, *val );

	// 辞書を破棄
	if ( ( ret = DADic_Close( dic ) ) != DA_OK )
		fprintf( stderr, "DADic_Close(): %d\n", ret );

	return	0;
}
