#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>

#include	"tinyda.h"


/*
	キー集合
*/


// マクロ


// ハッシュ表の初期サイズ ( 1 << HASH_MIN )
#define		HASH_MIN	8


// 構造体


// キーファイル
typedef		struct DAFile
{
	int		size;	// ファイルサイズ
	int		pos;	// 切り出し位置
	char	buf[];	// バッファ
}	DAFile, *hDAFile;

// キー
typedef		struct DAKey	*hDAKey;
typedef		struct DAKey
{
	hDAKey	next;	// 次のキー
	int		rec;	// レコード情報
	int		len;	// キー長
	int		idx;	// 終端のインデックス
	int		depth;	// 終端の深さ ( ルート直下を 0 とする )
	int		unify;	// 併合されるかどうか
	char	s[];	// キー
}	DAKey;

// キー集合
typedef		struct DASet
{
	hDAKey	*tbl;	// ハッシュ表
	int		nbit;	// ハッシュ表のサイズ ( size = 1 << nbit )
	int		mask;	// ハッシュ値のマスク ( mask = size - 1 )
	int		n;		// キー数
}	DASet;

// 整列済みキー集合
typedef		struct DASort
{
	int		n;		// キー数
	int		type;	// レコード情報の種類が複数あるかどうか
	int		max;	// レコード情報の最大値
	int		depth;	// 深さ ( キー長の最大値 )
	hDAKey	key[];	// キー集合
}	DASort, *hDASort;


// 文字列処理


// 長さ
static	int		DAStr_Len( const char *s )
{
	const char	*p = s;

	while ( *p != '\0' )
		p++;

	return	( int )( p - s );
}

// 複製
static	void	DAStr_Copy( char *dst, const char *src )
{
	do
		*dst++ = *src;
	while ( *src++ != '\0' );
}

// 比較
static	int		DAStr_Cmp( const char *a, const char *b )
{
	while ( *a != '\0' && *a == *b )
		a++, b++;

	return	( unsigned char )*a - ( unsigned char )*b;
}

// 一致確認
static	char	*DAStr_Match( char *a, const char *b )
{
	while ( *a != '\0' && *a == *b )
		a++, b++;

	return	( *a == *b ) ? ( a + 1 ) : NULL;
}

// 前方一致確認
static	char	*DAStr_Prefix( char *a, const char **b )
{
	while ( *a != '\0' && *a == **b )
		a++, ++*b;

	return	( *a == '\0' ) ? ( a + 1 ) : NULL;
}


// キーの作成と破棄


// 初期化
static	void	DAKey_Init( hDAKey h, const char *key, int len )
{
	memset( h, 0, sizeof( *h ) );

	h->next = NULL;
	h->len = len;

	DAStr_Copy( h->s, key );
}

// 作成
static	hDAKey	DAKey_Create( const char *key )
{
	int		len;
	hDAKey	h;

	len = DAStr_Len( key );
	if ( ( h = ( hDAKey )malloc( sizeof( *h ) + len + 1 ) ) != NULL )
		DAKey_Init( h, key, len );

	return	h;
}

// 終了処理
static	void	DAKey_Finish( hDAKey h )
{
}

// 破棄
static	void	DAKey_Delete( hDAKey h )
{
	DAKey_Finish( h );
	free( h );
}


// キー集合の作成と破棄


// 初期化
static	void	DASet_Init( hDASet h )
{
	memset( h, 0, sizeof( *h ) );
}

// 作成
static	hDASet	DASet_Create( void )
{
	hDASet	h;

	if ( ( h = ( hDASet )malloc( sizeof( *h ) ) ) != NULL )
		DASet_Init( h );

	return	h;
}

// 終了処理
static	void	DASet_Finish( hDASet h )
{
	int		i, max;
	hDAKey	key, next;
	int		ct, sum = 0;

	if ( h->tbl != NULL )
	{
		for ( i = 0, max = 1 << h->nbit; i < max; i++ )
		{
			ct = 0;
			for ( key = h->tbl[i]; key != NULL; key = next )
			{
				ct++;
				next = key->next;
				DAKey_Delete( key );
			}
			sum += ct * ct;
		}
		free( h->tbl );
	}
}

// 破棄
static	void	DASet_Delete( hDASet h )
{
	DASet_Finish( h );
	free( h );
}


// ハッシュ


// ハッシュ値の算出
static	int		DASet_Hash( const char *key )
{
	unsigned	val = 0;

	while ( *key != '\0' )
		val += ( val << 5 ) + *key++;

	return	( int )( val >> 1 );
}

// ハッシュ表の拡張にともなうキーの分配
static	void	DASet_Divide( hDASet h, int nbit )
{
	int		i, max, mask, val;
	hDAKey	*p, key;

	max = 1 << h->nbit;
	mask = ( 1 << nbit ) - 1;

	for ( i = 0; i < max; i++ )
	{
		// 拡張領域の初期化
		h->tbl[ max + i ] = NULL;
		for ( p = &h->tbl[i]; *p != NULL; )
		{
			// ハッシュ値の再計算
			if ( ( val = DASet_Hash( ( *p )->s ) & mask ) != i )
			{
				key = *p;
				*p = ( *p )->next;
				key->next = h->tbl[val];
				h->tbl[val] = key;
			}
			else
				p = &( *p )->next;
		}
	}
}

// ハッシュ表の拡張
static	int		DASet_Expand( hDASet h )
{
	int		i, max, nbit;
	hDAKey	*tbl;

	// サイズ計算と再割り当て
	nbit = ( h->nbit < HASH_MIN ) ? HASH_MIN : ( h->nbit + 1 );
	tbl = ( hDAKey * )realloc( h->tbl, sizeof( *tbl ) << nbit );
	if ( tbl == NULL )
		return	DA_EMEM;
	h->tbl = tbl;

	// 拡張領域の初期化とキーの分配
	if ( h->nbit < HASH_MIN )
	{
		for ( i = 0, max = 1 << nbit; i < max; i++ )
			tbl[i] = NULL;
	}
	else
		DASet_Divide( h, nbit );

	// 設定の反映
	h->nbit = nbit;
	h->mask = ( 1 << nbit ) - 1;

	return	DA_OK;
}

// ハッシュ検索
static	hDAKey	*DASet_Search( hDASet h, const char *key )
{
	int		val;
	hDAKey	*p;

	val = DASet_Hash( key ) & h->mask;
	for ( p = &h->tbl[val]; *p != NULL; p = &( *p )->next )
	{
		if ( !DAStr_Cmp( ( *p )->s, key ) )
			break;
	}

	return	p;
}

// ハッシュ表への追加
static	int		DASet_Add( hDASet h, const char *key, int rec )
{
	int		ret;
	hDAKey	*p;

	// ハッシュ表の拡張
	if ( h->n >= 1 << h->nbit )
	{
		if ( ( ret = DASet_Expand( h ) ) != DA_OK )
			return	ret;
	}

	// 未登録の場合は新規作成
	if ( *( p = DASet_Search( h, key ) ) == NULL )
	{
		// 新しいキーの作成
		if ( ( *p = DAKey_Create( key ) ) == NULL )
			return	DA_EMEM;
		h->n++;
	}

	// レコード情報の編集
	( *p )->rec = rec;

	return	DA_OK;
}


// キーファイルの操作


// 初期化
static	void	DAFile_Init( hDAFile h, int size )
{
	memset( h, 0, sizeof( *h ) );

	h->size = size;
}

// 作成
static	hDAFile		DAFile_Create( int size )
{
	hDAFile		h;

	if ( ( h = ( hDAFile )malloc( sizeof( *h ) + size + 1 ) ) != NULL )
		DAFile_Init( h, size );

	return	h;
}

// 終了処理
static	void	DAFile_Finish( hDAFile h )
{
}

// 破棄
static	void	DAFile_Delete( hDAFile h )
{
	DAFile_Finish( h );
	free( h );
}

// 読み込み
static	int		DAFile_Load( hDAFile h, FILE *fp )
{
	if ( fread( h->buf, h->size, 1, fp ) != 1 )
		return	DA_EIO;
	h->buf[ h->size ] = '\0';

	return	DA_OK;
}

// 領域の確保
static	int		DAFile_Alloc( hDAFile *h, FILE *fp )
{
	int		size;

	// 必要な領域の算出
	if ( fseek( fp, 0, SEEK_END ) == -1 ||
		 ( size = ftell( fp ) ) == -1 ||
		 fseek( fp, 0, SEEK_SET ) == -1 )
		return	DA_EIO;

	// 領域の確保
	if ( ( *h = DAFile_Create( size ) ) == NULL )
		return	DA_EMEM;

	return	DA_OK;
}

// 作成
static	int		DAFile_Open( hDAFile *h, const char *path )
{
	int		ret;
	FILE	*fp;

	// ファイルを開く
	if ( ( fp = fopen( path, "rb" ) ) == NULL )
		return	DA_EIO;

	if ( ( ret = DAFile_Alloc( h, fp ) ) == DA_OK )
	{
		if ( ( ret = DAFile_Load( *h, fp ) ) != DA_OK )
			DAFile_Delete( *h );
	}

	// ファイルを閉じる
	fclose( fp );

	return	ret;
}

// 破棄
static	void	DAFile_Close( hDAFile h )
{
	DAFile_Delete( h );
}

// 境界判定
static	int		DAFile_IsDelim( const char *delim, char c )
{
	while ( *delim != '\0' )
	{
		if ( c == *delim++ )
			return	1;
	}

	return	c == '\0';
}

// 切り出し
static	const char	*DAFile_Delim( hDAFile h, const char *delim )
{
	const char	*start;

	// キーの開始位置を取得
	while ( h->pos < h->size && DAFile_IsDelim( delim, h->buf[ h->pos ] ) )
		h->pos++;
	if ( h->pos >= h->size )
		return	NULL;
	start = h->buf + h->pos;

	// キーの終端を取得
	while ( !DAFile_IsDelim( delim, h->buf[ h->pos ] ) )
		h->pos++;
	h->buf[ h->pos++ ] = '\0';

	return	start;
}


// ファイル入力


// ファイルからキーを読み込んで追加
static	int		DASet_FileAdd( hDASet h, hDAFile file,
							   const char *delim, int rec )
{
	int		ret;
	const char	*key;

	while ( ( key = DAFile_Delim( file, delim ) ) != NULL )
	{
		if ( ( ret = DASet_Insert( h, key, rec ) ) != DA_OK )
			return	ret;
	}

	return	DA_OK;
}


// 整列の準備と破棄


// 初期化
static	void	DASort_Init( hDASort h, int n )
{
	memset( h, 0, sizeof( *h ) );

	h->n = n;
}

// 作成
static	hDASort		DASort_Create( int n )
{
	int		size;
	hDASort		h;

	size = sizeof( *h ) + sizeof( *h->key ) * n;
	if ( ( h = ( hDASort )malloc( size ) ) != NULL )
		DASort_Init( h, n );

	return	h;
}

// 終了処理
static	void	DASort_Finish( hDASort h )
{
}

// 破棄
static	void	DASort_Delete( hDASort h )
{
	DASort_Finish( h );
	free( h );
}

// 整列


// マルチキークイックソート
static	void	DASort_Quick( hDAKey *key, int n, int depth );


// 交換
static	void	DASort_Swap( hDAKey *key, int x, int y )
{
	hDAKey	tmp;

	tmp = key[x];
	key[x] = key[y];
	key[y] = tmp;
}

// 枢軸の選択
static	void	DASort_Pivot( hDAKey *key, int n, int depth )
{
	// 左端が中央より小さければ交換
	if ( ( unsigned char )key[0]->s[depth] <
		 ( unsigned char )key[ n >> 1 ]->s[depth] )
		DASort_Swap( key, 0, n >> 1 );

	// 右端が中央より小さければ，左端と中央を交換
	if ( ( unsigned char )key[ n - 1 ]->s[depth] <
		 ( unsigned char )key[ n >> 1 ]->s[depth] )
	{
		DASort_Swap( key, 0, n >> 1 );
		DASort_Swap( key, n >> 1, n - 1 );
	}
	// 右端が左端より小さければ，左端と右端を交換
	else if ( ( unsigned char )key[ n - 1 ]->s[depth] <
			  ( unsigned char )key[0]->s[depth] )
		DASort_Swap( key, 0, n - 1 );

	// 左端の次と中央を交換
	DASort_Swap( key, 1, n >> 1 );
}

// 挿入ソート
static	void	DASort_Insert( hDAKey *key, int n, int depth )
{
	int		i, j;
	const char	*s;

	for ( i = 1; i < n; i++ )
	{
		s = key[i]->s + depth;
		for ( j = i; j > 0; j-- )
		{
			if ( DAStr_Cmp( s, key[ j - 1 ]->s + depth ) > 0 )
				break;
			DASort_Swap( key, j - 1, j );
		}
	}
}

// マルチキークイックソートのサブルーチン
static	void	DASort_QuickSub( hDAKey *key, int n, int depth,
								 int l, unsigned char pivot )
{
	int		i, r, pl, pr, mv, diff;

	// 範囲の初期化
	pl = l;
	pr = r = n - 1;

	// 分割
	for ( ; ; )
	{
		// 枢軸と等しい要素を左端に移動
		while ( l <= r )
		{
			if ( ( diff = ( unsigned char )key[l]->s[depth] - pivot ) > 0 )
				break;
			else if ( !diff )
				DASort_Swap( key, l, pl++ );
			l++;
		}

		// 枢軸と等しい要素を右端に移動
		while ( l <= r )
		{
			if ( ( diff = ( unsigned char )key[r]->s[depth] - pivot ) < 0 )
				break;
			else if ( !diff )
				DASort_Swap( key, r, pr-- );
			r--;
		}

		if ( l > r )
			break;

		// 左右の交換
		DASort_Swap( key, l++, r-- );
	}

	// 左側の枢軸と等しい要素を中央に移動
	mv = ( pl < l - pl ) ? pl : l - pl;
	for ( i = 0; i < mv; i++ )
		DASort_Swap( key, i, r - i );
	pl = l - pl;

	// 右側の枢軸と等しい要素を中央に移動
	n--;
	mv = ( n - pr < pr - r ) ? n - pr : pr - r;
	for ( i = 0; i < mv; i++ )
		DASort_Swap( key, l + i, n - i );
	pr = ++n - ( pr - r );

	// 再帰的に整列
	if ( pl > 1 )
		DASort_Quick( key, pl, depth );
	if ( pr - pl > 1 )
		DASort_Quick( key + pl, pr - pl, depth + 1 );
	if ( n - pr > 1 )
		DASort_Quick( key + pr, n - pr, depth );
}

// マルチキークイックソート
static	void	DASort_Quick( hDAKey *key, int n, int depth )
{
	int		i;
	unsigned char	pivot;

	if ( n < 8 )
	{
		// 少なくなれば挿入ソート
		DASort_Insert( key, n, depth );
		return;
	}

	depth--;
	do
	{
		// 枢軸を左端に移動する
		DASort_Pivot( key, n, ++depth );
		pivot = ( unsigned char )key[0]->s[depth];

		// 探索範囲を求める
		for ( i = 1; i < n; i++ )
		{
			if ( ( unsigned char )key[i]->s[depth] != pivot )
				break;
		}
	}	while ( i == n );

	// サブルーチン
	DASort_QuickSub( key, n, depth, i, pivot );
}

// ビンソート
static	void	DASort_Bin( hDAKey *key, hDASet set )
{
	int		i, max, depth;
	int		buf[258], *freq;
	hDAKey	p;

	// 初期設定
	depth = 0;
	max = 1 << set->nbit;

	do
	{
		// 深さ depth の文字の頻度を計算
		freq = buf + 2;
		memset( buf, 0, sizeof( buf ) );
		for ( i = 0; i < max; i++ )
		{
			for ( p = set->tbl[i]; p != NULL; p = p->next )
				freq[ ( unsigned char )p->s[depth] ]++;
		}

		// 深さ depth の文字の頻度を合計
		for ( i = 1, freq--; i < 256; i++ )
		{
			if ( freq[i] == set->n && set->n > 1 )
			{
				depth++;
				break;
			}
			freq[i] += freq[ i - 1 ];
		}
	}	while ( i < 256 );

	// 深さ depth の文字に応じて分配
	for ( i = 0; i < max; i++ )
	{
		for ( p = set->tbl[i]; p != NULL; p = p->next )
			key[ freq[ ( unsigned char )p->s[depth] ]++ ] = p;
	}

	// 深さ depth 以降に応じて整列
	for ( i = 0, freq--, depth++; i < 256; i++ )
	{
		if ( freq[ i + 1 ] - freq[i] > 1 )
			DASort_Quick( key + freq[i], freq[ i + 1 ] - freq[i], depth );
	}
}

// レコード情報とキー長の確認
static	void	DASort_Check( hDASort h )
{
	int		i;

	// 初期値
	h->max = h->key[0]->rec;
	h->depth = h->key[0]->len;

	for ( i = 1; i < h->n; i++ )
	{
		// レコード情報の最大値
		if ( h->key[i]->rec > h->max )
			h->max = h->key[i]->rec;

		// レコード情報の種類
		if ( h->key[ i - 1 ]->rec != h->key[i]->rec )
			h->type = 1;

		// 最大キー長
		if ( h->key[i]->len > h->depth )
			h->depth = h->key[i]->len;
	}
}


// 中間インタフェース


// 整列キー集合の作成
static	int		DASort_Open( hDASort *h, hDASet set )
{
	// 領域を確保
	if ( ( *h = DASort_Create( set->n ) ) == NULL )
		return	DA_EMEM;

	// ビンソートしてからマルチキークイックソート
	DASort_Bin( ( *h )->key, set );

	// レコード情報の確認
	DASort_Check( *h );

	return	DA_OK;
}

// 整列キー集合の破棄
static	void	DASort_Close( hDASort h )
{
	DASort_Delete( h );
}


// サフィックス降順に整列


// マルチキークイックソート
static	void	DASuffix_Quick( hDAKey *key, int n, int depth );


// サフィックスの文字
static	int		DASuffix_Char( hDAKey key, int depth )
{
	if ( depth >= key->len - key->depth )
		return	'\0';

	return	( unsigned char )key->s[ key->len - depth - 1 ];
}

// サフィックスの比較
static	int		DASuffix_Cmp( hDAKey a, hDAKey b, int depth )
{
	int		ac, bc;

	do
	{
		ac = DASuffix_Char( a, depth );
		bc = DASuffix_Char( b, depth++ );
	}	while ( ac != '\0' && ac == bc );

	return	ac - bc;
}

// 併合可能かどうか
static	int		DASuffix_Part( hDAKey prev, hDAKey key )
{
	int		ac, bc, depth = 0;

	do
	{
		ac = DASuffix_Char( prev, depth );
		bc = DASuffix_Char( key, depth++ );
	}	while ( ac != '\0' && ac == bc );

	return	bc == '\0';
}

// 同じキーを併合
static	void	DASuffix_Unify( hDAKey *key, int n )
{
	int		i;

	for ( i = 0; i < n; i++ )
		key[i]->unify = 1;
}

// 枢軸の選択
static	void	DASuffix_Pivot( hDAKey *key, int n, int depth )
{
	// 左端が中央より大きければ交換
	if ( DASuffix_Char( key[0], depth ) >
		 DASuffix_Char( key[ n >> 1 ], depth ) )
		DASort_Swap( key, 0, n >> 1 );

	// 右端が中央より大きければ，左端と中央を交換
	if ( DASuffix_Char( key[ n - 1 ], depth ) >
		 DASuffix_Char( key[ n >> 1 ], depth ) )
	{
		DASort_Swap( key, 0, n >> 1 );
		DASort_Swap( key, n >> 1, n - 1 );
	}
	// 右端が左端より大きければ，左端と右端を交換
	else if ( DASuffix_Char( key[ n - 1 ], depth ) >
			  DASuffix_Char( key[0], depth ) )
		DASort_Swap( key, 0, n - 1 );

	// 左端の次と中央を交換
	DASort_Swap( key, 1, n >> 1 );
}

// 挿入ソート
static	void	DASuffix_Insert( hDAKey *key, int n, int depth )
{
	int		i, j;

	for ( i = 1; i < n; i++ )
	{
		for ( j = i; j > 0; j-- )
		{
			if ( DASuffix_Cmp( key[j], key[ j - 1 ], depth ) < 0 )
				break;
			DASort_Swap( key, j - 1, j );
		}
	}
}

// マルチキークイックソートのサブルーチン
static	void	DASuffix_QuickSub( hDAKey *key, int n, int depth,
								   int l, int pivot )
{
	int		i, r, pl, pr, mv, diff;

	// 範囲の初期化
	pl = l;
	pr = r = n - 1;

	// 分割
	for ( ; ; )
	{
		// 枢軸と等しい要素を左端に移動
		while ( l <= r )
		{
			if ( ( diff = DASuffix_Char( key[l], depth ) - pivot ) < 0 )
				break;
			else if ( !diff )
				DASort_Swap( key, l, pl++ );
			l++;
		}

		// 枢軸と等しい要素を右端に移動
		while ( l <= r )
		{
			if ( ( diff = DASuffix_Char( key[r], depth ) - pivot ) > 0 )
				break;
			else if ( !diff )
				DASort_Swap( key, r, pr-- );
			r--;
		}

		if ( l > r )
			break;

		// 左右の交換
		DASort_Swap( key, l++, r-- );
	}

	// 左側の枢軸と等しい要素を中央に移動
	mv = ( pl < l - pl ) ? pl : l - pl;
	for ( i = 0; i < mv; i++ )
		DASort_Swap( key, i, r - i );
	pl = l - pl;

	// 右側の枢軸と等しい要素を中央に移動
	n--;
	mv = ( n - pr < pr - r ) ? n - pr : pr - r;
	for ( i = 0; i < mv; i++ )
		DASort_Swap( key, l + i, n - i );
	pr = ++n - ( pr - r );

	// 再帰的に整列
	if ( pl > 1 )
		DASuffix_Quick( key, pl, depth );
	if ( pr - pl > 1 )
	{
		if ( pivot != '\0' )
			DASuffix_Quick( key + pl, pr - pl, depth + 1 );
		else
			DASuffix_Unify( key + pl, pr - pl );
	}
	if ( n - pr > 1 )
		DASuffix_Quick( key + pr, n - pr, depth );
}

// マルチキークイックソート
static	void	DASuffix_Quick( hDAKey *key, int n, int depth )
{
	int		i, pivot;

	if ( n < 8 )
	{
		// 少なくなれば挿入ソート
		DASuffix_Insert( key, n, depth );
		return;
	}

	depth--;
	do
	{
		// 枢軸を左端に移動する
		DASuffix_Pivot( key, n, ++depth );
		pivot = DASuffix_Char( key[0], depth );

		// 探索範囲を求める
		for ( i = 1; i < n; i++ )
		{
			if ( DASuffix_Char( key[i], depth ) != pivot )
				break;
		}
	}	while ( i == n && pivot != '\0' );

	// サブルーチン
	if ( i < n )
		DASuffix_QuickSub( key, n, depth, i, pivot );
	else
		DASuffix_Unify( key + 1, n - 1 );
}

// 併合可能なサフィックスを調査
static	void	DASuffix_Check( hDASort h )
{
	int		i;

	for ( i = 1; i < h->n; i++ )
	{
		// 直前のサフィックスで併合可能かどうかを確認
		if ( !h->key[i]->unify )
			h->key[i]->unify = DASuffix_Part( h->key[ i - 1 ], h->key[i] );
	}
}

// サフィックス降順に整列
static	void	DASuffix_Sort( hDASort h )
{
	// マルチキークイックソート
	DASuffix_Quick( h->key, h->n, 0 );

	// 併合可能なサフィックスを調査
	DASuffix_Check( h );
}


// 外部インタフェース


// キー集合の作成
int		DASet_Open( hDASet *h )
{
	int		ret;

	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;

	// 構造体の作成
	if ( ( *h = DASet_Create() ) == NULL )
		return	DA_EMEM;

	// ハッシュ表の作成
	if ( ( ret = DASet_Expand( *h ) ) != DA_OK )
	{
		DASet_Delete( *h );
		*h = NULL;
	}

	return	ret;
}

// キー集合の読み込み
int		DASet_Load( hDASet h, const char *path, const char *delim, int rec )
{
	int		ret;
	hDAFile		file;

	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;
	else if ( path == NULL || delim == NULL )
		return	DA_EARG;

	// キーファイルを開く
	if ( ( ret = DAFile_Open( &file, path ) ) != DA_OK )
		return	ret;

	// ファイルからキーを読み込んで追加
	ret = DASet_FileAdd( h, file, delim, rec );

	// キーファイルを破棄
	DAFile_Close( file );

	return	DA_OK;
}

// キーの追加
int		DASet_Insert( hDASet h, const char *key, int rec )
{
	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;
	else if ( key == NULL )
		return	DA_EARG;

	// 引数を補正
	if ( rec < 0 )
		rec = 0;

	return	DASet_Add( h, key, rec );
}

// キー集合の破棄
int		DASet_Close( hDASet h )
{
	if ( h == NULL )
		return	DA_EHDL;

	// 構造体の破棄
	DASet_Delete( h );

	return	DA_OK;
}


/*
	辞書
*/


// マクロ


// バージョン情報
#define		DA_VERSION	100

// 構築用の一時情報
#define		FLAG_USED	( 1 << 0 )	// 使用要素フラグ
#define		FLAG_BASE	( 1 << 1 )	// BASE 使用済みフラグ

// サイズ上限
#define		BC_MAX		0x00800000	//     8,388,608
#define		TAIL_MAX	0x7FFFFFFF	// 2,147,483,647

// ルートのインデックス
#define		IDX_ROOT	0

// BC の内容
#define		VAL_LEAF	0x00000001U
#define		VAL_BASE	0x007FFFFFU
#define		VAL_CHECK	0x000000FFU
#define		VAL_LINK	0x7FFFFFFFU
#define		OFF_LEAF	31
#define		OFF_BASE	8
#define		OFF_CHECK	0
#define		OFF_LINK	0
#define		MASK_LEAF	( VAL_LEAF << OFF_LEAF )
#define		MASK_BASE	( VAL_BASE << OFF_BASE )
#define		MASK_CHECK	( VAL_CHECK << OFF_CHECK )
#define		MASK_LINK	( VAL_LINK << OFF_LINK )

// ヘッダ識別用文字列
static	const char	HEAD_STR[8] = "TDA";


// バイト境界
#define		BOUND_MIN	( 1 << 0 )	//	1 バイト
#define		BOUND_MAX	( 1 << 16 )	//	65,536 バイト


// 構造体


// 要素
typedef		unsigned	DAInt, *hDAInt;

// 遷移
typedef		struct DAArc
{
	int		c;	// 遷移文字
	int		n;	// 頻度
}	DAArc, *hDAArc;

// 分岐
typedef		struct DASib
{
	DAArc	arc[256];	// 遷移
	int		n;			// 遷移数
}	DASib, *hDASib;

// 構築用の一時情報
typedef		struct DAInf
{
	hDADic		dic;	// 辞書
	hDASort		sort;	// 整列キー集合
	hDASib		sib;	// 分岐
	char		*flag;	// BASE 使用済みフラグと使用要素フラグ
	int			n;		// フラグ用配列の長さ ( 配列 BC と同じ )
	int			ntail;	// 配列 TAIL の長さ
	int			max;	// 末尾の要素
	int			empty;	// 先頭の未使用要素
	int			pos;	// TAIL の追加位置
	int			mask;	// バイト境界のマスク
}	DAInf, *hDAInf;

// 辞書のヘッダ
typedef		struct DAHead
{
	char	str[4];		// 識別用文字列
	int		version;	// バージョン情報
	int		nbc;		// 配列 BC の長さ
	int		ntail;		// 配列 TAIL の長さ
	int		nkey;		// キー数
	int		nbit;		// 併合用の下位ビット数
	int		boundary;	// バイト境界
	int		rec;		// レコード情報の配置間隔
}	DAHead, *hDAHead;

// 辞書
typedef		struct DADic
{
	hDAInt	bc;			// 配列 BC ( BASE, CHECK )
	char	*tail;		// 配列 TAIL
	int		nbc;		// 配列 BC の長さ
	int		ntail;		// 配列 TAIL の長さ
	int		nkey;		// キー数
	int		nbit;		// 併合用の下位ビット数
	DAInt	hmask;		// 上位マスク
	DAInt	lmask;		// 下位マスク
	int		boundary;	// バイト境界
	int		rec;		// レコード情報の配置間隔
}	DADic;


// 要素の操作


// LEAF 読み込み
static	int		DAInt_Leaf( hDAInt h )
{
	return	( int )( ( *h & MASK_LEAF ) >> OFF_LEAF );
}

// LEAF 書き込み
static	void	DAInt_WLeaf( hDAInt h, int val )
{
	*h &= ~MASK_LEAF;
	*h |= ( ( DAInt )val & VAL_LEAF ) << OFF_LEAF;
}

// BASE 読み込み
static	int		DAInt_Base( hDAInt h )
{
	return	( int )( ( *h & MASK_BASE ) >> OFF_BASE );
}

// BASE 書き込み
static	void	DAInt_WBase( hDAInt h, int val )
{
	*h &= ~MASK_BASE;
	*h |= ( ( DAInt )val & VAL_BASE ) << OFF_BASE;
}

// CHECK 読み込み
static	int		DAInt_Check( hDAInt h )
{
	return	( int )( ( *h & MASK_CHECK ) >> OFF_CHECK );
}

// CHECK 書き込み
static	void	DAInt_WCheck( hDAInt h, int val )
{
	*h &= ~MASK_CHECK;
	*h |= ( ( DAInt )val & VAL_CHECK ) << OFF_CHECK;
}


// 辞書の作成と破棄


// 初期化
static	void	DADic_Init( hDADic h )
{
	memset( h, 0, sizeof( *h ) );

	h->bc = NULL;
	h->tail = NULL;
}

// 作成
static	hDADic	DADic_Create( void )
{
	hDADic	h;

	if ( ( h = ( hDADic )malloc( sizeof( *h ) ) ) != NULL )
		DADic_Init( h );

	return	h;
}

// 終了処理
static	void	DADic_Finish( hDADic h )
{
	if ( h->bc != NULL )
		free( h->bc );
	if ( h->tail != NULL )
		free( h->tail );
}

// 破棄
static	void	DADic_Delete( hDADic h )
{
	DADic_Finish( h );
	free( h );
}

// 配列 BC のサイズ変更
static	int		DADic_ResizeBC( hDADic h, int n )
{
	hDAInt	bc;

	// 領域を再割り当て
	if ( ( bc = ( hDAInt )realloc( h->bc, sizeof( *bc ) * n ) ) == NULL )
		return	DA_EMEM;

	h->bc = bc;
	h->nbc = n;

	return	DA_OK;
}

// 配列 TAIL のサイズ変更
static	int		DADic_ResizeTail( hDADic h, int n )
{
	char	*tail;

	// 領域を再割り当て
	if ( ( tail = ( char * )realloc( h->tail, n ) ) == NULL )
		return	DA_EMEM;

	h->tail = tail;
	h->ntail = n;

	return	DA_OK;
}


// 辞書の操作


// LEAF 読み込み
static	int		DADic_Leaf( hDADic h, int idx )
{
	return	DAInt_Leaf( h->bc + idx );
}

// LEAF 書き込み
static	void	DADic_WLeaf( hDADic h, int idx, int val )
{
	DAInt_WLeaf( h->bc + idx, val );
}

// BASE 読み込み
static	int		DADic_Base( hDADic h, int idx )
{
	return	DAInt_Base( h->bc + idx );
}

// BASE 書き込み
static	void	DADic_WBase( hDADic h, int idx, int val )
{
	DAInt_WBase( h->bc + idx, val );
}

// CHECK 読み込み
static	int		DADic_Check( hDADic h, int idx )
{
	return	DAInt_Check( h->bc + idx );
}

// CHECK 書き込み
static	void	DADic_WCheck( hDADic h, int idx, int val )
{
	DAInt_WCheck( h->bc + idx, val );
}

// POS 読み込み
static	int		DADic_Pos( hDADic h, int idx )
{
	return	( int )( h->bc[idx] & h->hmask ) >> h->nbit;
}

// POS 書き込み
static	void	DADic_WPos( hDADic h, int idx, int val )
{
	h->bc[idx] &= ~h->hmask;
	h->bc[idx] |= ( ( DAInt )val << h->nbit ) & h->hmask;
}

// REC 読み込み
static	int		DADic_Rec( hDADic h, int idx )
{
	return	( int )( h->bc[idx] & h->lmask );
}

// REC 書き込み
static	void	DADic_WRec( hDADic h, int idx, int val )
{
	h->bc[idx] &= ~h->lmask;
	h->bc[idx] |= ( DAInt )val & h->lmask;
}

// TAIL 読み込み
static	char	*DADic_Str( hDADic h, int pos )
{
	return	h->tail + pos;
}

// TAIL 書き込み
static	void	DADic_WStr( hDADic h, int pos, const char *s )
{
	DAStr_Copy( h->tail + pos, s );
}

// マスクを生成
static	void	DADic_SetMask( hDADic h, int nbit )
{
	h->nbit = nbit;
	h->lmask = ( ( DAInt )1 << nbit ) - 1;
	h->hmask = MASK_LINK & ~h->lmask;
}


// 構築用の一時情報の作成と破棄


// 初期化
static	void	DAInf_Init( hDAInf h, hDADic dic, hDASort sort )
{
	memset( h, 0, sizeof( *h ) );

	h->dic = dic;
	h->sort = sort;
	h->sib = NULL;
	h->flag = NULL;
}

// 作成
static	hDAInf	DAInf_Create( hDADic dic, hDASort sort )
{
	hDAInf	h;

	if ( ( h = ( hDAInf )malloc( sizeof( *h ) ) ) != NULL )
		DAInf_Init( h, dic, sort );

	return	h;
}

// 終了処理
static	void	DAInf_Finish( hDAInf h )
{
	if ( h->flag != NULL )
		free( h->flag );
	if ( h->sib != NULL )
		free( h->sib );
}

// 破棄
static	void	DAInf_Delete( hDAInf h )
{
	DAInf_Finish( h );
	free( h );
}


// 分岐を取得


// 分岐を取得
static	void	DASib_Scan( hDASib h, hDAKey *key, int n, int depth )
{
	int		i, type;

	// 最初の遷移
	h->arc[0].c = ( unsigned char )key[0]->s[depth];
	h->arc[0].n = 1;

	// 各遷移の重複数
	for ( i = 1, type = 0; i < n; i++ )
	{
		if ( h->arc[type].c == ( unsigned char )key[i]->s[depth] )
			h->arc[type].n++;
		else
		{
			h->arc[ ++type ].c = ( unsigned char )key[i]->s[depth];
			h->arc[type].n = 1;
		}
	}

	// 遷移の種類
	h->n = type + 1;
}


// 構築用の領域操作


// 分岐用の領域を確保
static	int		DAInf_ResizeSib( hDAInf h, int n )
{
	hDASib	sib;

	if ( ( sib = ( hDASib )realloc( h->sib, sizeof( *sib ) * n ) ) == NULL )
		return	DA_EMEM;
	h->sib = sib;

	return	DA_OK;
}

// 分岐用の領域を確保
static	int		DAInf_ExpandSib( hDAInf h )
{
	return	DAInf_ResizeSib( h, h->sort->depth );
}

// 拡張領域の初期化
static	void	DAInf_ClearBC( hDAInf h, int n )
{
	int		i;
	hDAInt	bc = h->dic->bc;

	if ( n > h->n )
	{
		// フラグを初期化
		memset( h->flag + h->n, 0, n - h->n );

		// 未使用要素を連結
		for ( i = h->n; i < n; i++ )
			bc[i] = i + 1;
	}

	h->n = n;
}

// 再割り当て
static	int		DAInf_ResizeBC( hDAInf h, int n )
{
	int		ret;
	char	*flag;

	// 領域の再割り当て
	if ( ( flag = ( char * )realloc( h->flag, n ) ) == NULL )
		return	DA_EMEM;
	h->flag = flag;

	// 配列 BC のサイズ変更
	if ( ( ret = DADic_ResizeBC( h->dic, n ) ) != DA_OK )
		return	ret;

	// 拡張領域の初期化
	DAInf_ClearBC( h, n );

	return	DA_OK;
}

// 拡張
static	int		DAInf_ExpandBC( hDAInf h )
{
	int		size;

	// サイズ上限
	if ( h->n >= BC_MAX )
		return	DA_EMAX;

	// 拡張後のサイズ
	size = h->n ? h->n : h->sort->n;
	size = ( ( size + ( size >> 1 ) ) + 0x100 ) & ~0xFF;
	if ( size > BC_MAX )
		size = BC_MAX;

	return	DAInf_ResizeBC( h, size );
}

// 縮小
static	int		DAInf_ReduceBC( hDAInf h )
{
	int		size;

	size = ( h->max + 0x100 ) & ~0xFF;

	return	DAInf_ResizeBC( h, size );
}

// 拡張領域の初期化
static	void	DAInf_ClearTail( hDAInf h, int n )
{
	if ( n > h->ntail )
		memset( h->dic->tail + h->ntail, 0, n - h->ntail );

	h->ntail = n;
}

// 再割り当て
static	int		DAInf_ResizeTail( hDAInf h, int n )
{
	int		ret;

	// 配列 TAIL のサイズ変更
	if ( ( ret = DADic_ResizeTail( h->dic, n ) ) != DA_OK )
		return	ret;

	// 拡張領域の初期化
	DAInf_ClearTail( h, n );

	return	DA_OK;
}

// 縮小
static	int		DAInf_ReduceTail( hDAInf h )
{
	// 必要なし
	if ( h->pos == h->ntail )
		return	DA_OK;

	return	DADic_ResizeTail( h->dic, h->pos );
}


// 構築用のサブルーチン


// 使用要素かどうかを確認
static	int		DAInf_Used( hDAInf h, int idx )
{
	return	h->flag[idx] & FLAG_USED;
}

// BASE 使用済みかどうかを確認
static	int		DAInf_Base( hDAInf h, int idx )
{
	return	( h->flag[idx] & FLAG_BASE ) == FLAG_BASE;
}

// 次の未使用要素
static	int		DAInf_NextEmpty( hDAInf h, int idx )
{
	int		next;

	// 次の未使用要素の候補
	next = h->dic->bc[idx];

	// 後方に未使用要素を探索
	while ( next < h->n && DAInf_Used( h, next ) )
		next++;

	// 候補を更新
	h->dic->bc[idx] = next;

	return	next;
}

// 使用要素フラグを設定
static	void	DAInf_WUsed( hDAInf h, int idx )
{
	// 先頭の未使用要素を更新
	if ( idx == h->empty )
		h->empty = DAInf_NextEmpty( h, idx );

	// 末尾の要素を更新
	if ( idx > h->max )
		h->max = idx;

	h->flag[idx] |= FLAG_USED;
}

// BASE 使用済みフラグを設定
static	void	DAInf_WBase( hDAInf h, int idx )
{
	h->flag[idx] |= FLAG_BASE;
}

// ルートを作成
static	int		DAInf_Root( hDAInf h )
{
	int		ret;

	// 初期領域を確保 ( 経験的にキー数の約 1.5 倍 )
	if ( ( ret = DAInf_ExpandBC( h ) ) != DA_OK )
		return	ret;

	// ルートを確保
	DAInf_WUsed( h, IDX_ROOT );
	DAInf_WBase( h, IDX_ROOT );
	DADic_WLeaf( h->dic, IDX_ROOT, 0 );
	DADic_WBase( h->dic, IDX_ROOT, 0 );
	DADic_WCheck( h->dic, IDX_ROOT, 0 );

	return	DA_OK;
}


// BC の構築


// 適切な BASE を探索
static	int		DAInf_Search( hDAInf h, hDASib sib )
{
	int		i, empty, base;

	// 未使用要素を探索
	for ( empty = h->empty; empty < h->n; empty = DAInf_NextEmpty( h, empty ) )
	{
		// BASE 使用済みかどうか
		base = empty ^ sib->arc[0].c;
		if ( DAInf_Base( h, base ) )
			continue;

		// 分岐が収納可能かどうか
		for ( i = 1; i < sib->n; i++ )
		{
			if ( DAInf_Used( h, base ^ sib->arc[i].c ) )
				break;
		}

		// 分岐が収納可能
		if ( i == sib->n )
			return	base;
	}

	return	empty ^ sib->arc[0].c;
}

// 分岐を配置
static	int		DAInf_Arrange( hDAInf h, hDASib sib, int idx )
{
	int		i, base, next, ret;

	// 配置先を取得
	if ( ( base = DAInf_Search( h, sib ) ) >= h->n )
	{
		// 領域を拡張
		if ( ( ret = DAInf_ExpandBC( h ) ) != DA_OK )
			return	ret;
	}

	// BASE を設定
	DAInf_WBase( h, base );
	DADic_WBase( h->dic, idx, base );

	for ( i = 0; i < sib->n; i++ )
	{
		// CHECK を設定
		next = base ^ sib->arc[i].c;
		DAInf_WUsed( h, next );
		DADic_WCheck( h->dic, next, sib->arc[i].c );
	}

	return	DA_OK;
}

// 深さ優先に構築
static	int		DAInf_Progress( hDAInf h, hDAKey *key, int n,
								int idx, int depth )
{
	int		i, single, base, ret;
	hDASib	sib;

	// 終端
	single = depth && ( n == 1 );
	DADic_WLeaf( h->dic, idx, single );
	if ( single )
	{
		key[0]->idx = idx;
		key[0]->depth = depth - 1;
		return DA_OK;
	}

	// 分岐を取得
	sib = h->sib + depth;
	DASib_Scan( sib, key, n, depth );

	// 配置
	if ( ( ret = DAInf_Arrange( h, sib, idx ) ) != DA_OK )
		return	ret;
	base = DADic_Base( h->dic, idx );

	for ( i = 0; i < sib->n; i++ )
	{
		// 再帰的に構築
		ret = DAInf_Progress( h, key, sib->arc[i].n,
							  base ^ sib->arc[i].c, depth + 1 );
		if ( ret != DA_OK )
			return	ret;

		key += sib->arc[i].n;
	}

	return	DA_OK;
}

// BC を構築
static	int		DAInf_BuildBC( hDAInf h )
{
	int		ret;

	// 空の辞書
	if ( h->sort->n <= 0 )
		return	DA_OK;

	// 分岐用の領域を確保
	if ( ( ret = DAInf_ExpandSib( h ) ) != DA_OK )
		return	ret;

	// 深さ優先に構築
	ret = DAInf_Progress( h, h->sort->key, h->sort->n, IDX_ROOT, 0 );
	if ( ret != DA_OK )
		return	ret;

	return	DA_OK;
}

// 未使用要素を修正
static	void	DAInf_Revise( hDAInf h )
{
	int		empty, next, base = 0, block = -1;

	for ( empty = h->empty; empty < h->n; empty = next )
	{
		next = DAInf_NextEmpty( h, empty );

		// ブロック中で未使用の BASE を探索
		if ( empty >> 8 != block )
		{
			block = empty >> 8;
			base = block << 8;
			while ( DAInf_Used( h, base ) )
				base++;
		}

		// 未使用要素の設定
		DADic_WLeaf( h->dic, empty, 0 );
		DADic_WBase( h->dic, empty, 0 );
		DADic_WCheck( h->dic, empty, ( base ^ empty ) & 0xFF );
	}
}


// TAIL の構築


// 併合するかどうか
static	int		DAInf_NeedUnify( hDAInf h )
{
	if ( h->sort->max <= h->dic->boundary )
		return	1;

	return	!h->sort->type;
}

// バイト境界を適用するかどうか
static	int		DAInf_NeedBoundary( hDAInf h, int no )
{
	int		max;
	hDAKey	*key;

	max = h->sort->max;
	key = h->sort->key;
	do
	{
		// レコード有り
		if ( key[no]->rec )
			return	1;
	}	while ( ++no < max && key[no]->unify );

	// レコードなし
	return	0;
}

// レコードの配置間隔を計算
static	void	DAInf_RecordPeriod( hDAInf h )
{
	h->mask = h->dic->boundary - 1;

	// サフィックスを併合するとき
	if ( DAInf_NeedUnify( h ) )
		h->dic->rec = ( h->sort->max + h->mask ) & ~h->mask;

}

// TAIL の領域を計算して確保
static	int		DAInf_TotalTail( hDAInf h )
{
	int		i, max, len, pos;
	hDAKey	*key;

	max = h->sort->n;
	key = h->sort->key;
	for ( i = pos = 0; i < max; i++ )
	{
		// サフィックスの長さ
		len = key[i]->len - key[i]->depth + 1;

		if ( key[i]->rec )
		{
			// レコードがあるのでバイト境界を適用
			if ( pos > TAIL_MAX - len - h->mask )
				return	DA_EMAX;
			pos = ( pos + len + h->mask ) & ~h->mask;
			if ( pos > TAIL_MAX - key[i]->rec )
				return	DA_EMAX;
			pos += ( h->dic->rec > key[i]->rec ) ? h->dic->rec : key[i]->rec;
		}
		else
		{
			// レコードがないのでバイト境界を無視
			if ( pos > TAIL_MAX - len )
				return	DA_EMAX;
			pos += len;
		}
	}

	// 領域を確保
	return	DAInf_ResizeTail( h, pos );
}

// サフィックスを列挙
static	void	DAInf_LineTail( hDAInf h )
{
	int		i, max, len;
	hDAKey	*key;

	// マスクを作成
	DADic_SetMask( h->dic, 0 );

	max = h->sort->n;
	key = h->sort->key;
	for ( i = 0; i < max; i++ )
	{
		// サフィックスの長さ
		len = key[i]->len - key[i]->depth + 1;

		// レコードのバイト境界を考慮
		if ( key[i]->rec )
			h->pos = ( ( h->pos + len + h->mask ) & ~h->mask ) - len;

		// サフィックスとレコードの配置
		DADic_WStr( h->dic, h->pos, key[i]->s + key[i]->depth );
		DADic_WPos( h->dic, key[i]->idx, h->pos );
		DADic_WRec( h->dic, key[i]->idx, 0 );

		h->pos += len + key[i]->rec;
	}
}

// 共通サフィックスを併合
static	void	DAInf_UnifyTail( hDAInf h )
{
	int		i, max, len, ct, nbit = 0;
	hDAKey	*key;

	// サフィックス降順に整列
	DASuffix_Sort( h->sort );

	// マスクを作成
	while ( h->ntail >> nbit )
		nbit++;
	DADic_SetMask( h->dic, 31 - nbit );

	// マスクに応じて併合を制限しつつ配置
	max = h->sort->n;
	key = h->sort->key;
	for ( i = ct = 0; i < max; i++ )
	{
		// サフィックスの長さ
		len = key[i]->len - key[i]->depth + 1;

		// サフィックスの併合を制限
		if ( key[i]->unify && key[i]->rec && ct == h->dic->lmask )
			key[i]->unify = 0;

		// 併合できない場合
		if ( !key[i]->unify )
		{
			// 追加位置を移動
			h->pos += h->dic->rec * ct;
			ct = 0;

			// レコードのバイト境界を考慮
			if ( DAInf_NeedBoundary( h, i ) )
				h->pos = ( h->pos + len + h->mask ) & ~h->mask;
			else
				h->pos += len;

			// サフィックスの書き込み
			DADic_WStr( h->dic, h->pos - len, key[i]->s + key[i]->depth );
		}

		// サフィックスとレコードの配置
		DADic_WPos( h->dic, key[i]->idx, h->pos - len );
		DADic_WRec( h->dic, key[i]->idx, ct );

		// レコードの有無
		if ( key[i]->rec )
			ct++;
	}

	// 最後のレコード
	h->pos += h->dic->rec * ct;
}

// TAIL を構築
static	int		DAInf_BuildTail( hDAInf h )
{
	int		ret;

	// レコードの配置間隔を計算
	DAInf_RecordPeriod( h );

	// 領域を確保
	if ( ( ret = DAInf_TotalTail( h ) ) != DA_OK )
		return	ret;

	// サフィックスとレコードの配置
	if ( DAInf_NeedUnify( h ) )
		DAInf_UnifyTail( h );
	else
		DAInf_LineTail( h );

	// 領域を調整
	if ( ( ret = DAInf_ReduceTail( h ) ) != DA_OK )
		return	ret;

	return	DA_OK;
}


// 辞書の構築


// 境界の設定
static	int		DADic_Boundary( hDADic h, int boundary )
{
	int		i;
	unsigned	x, n = 0;

	// 範囲の確認
	if ( boundary == 0 )
		boundary = BOUND_MIN;
	else if ( boundary < BOUND_MIN  || boundary > BOUND_MAX )
		return	DA_EARG;

	// 2 の累乗になっているか確認する
	x = ( unsigned )boundary;
	for ( i = 0; i < 32; i++ )
		n += x = ( x >> 1 ) | ( x << 31 );
	if ( ~n )
		return	DA_EARG;

	h->boundary = boundary;

	return	DA_OK;
}

// 編集
static	int		DADic_Edit( hDADic h, hDASort sort )
{
	int		ret;
	hDAInf	inf;

	// 構築用の一時情報を作成
	if ( ( inf = DAInf_Create( h, sort ) ) == NULL )
		return	DA_EMEM;

	// ルート，BC，TAIL の順に構築
	if ( ( ret = DAInf_Root( inf ) ) == DA_OK &&
		 ( ret = DAInf_BuildBC( inf ) ) == DA_OK &&
		 ( ret = DAInf_ReduceBC( inf ) ) == DA_OK )
	{
		DAInf_Revise( inf );
		ret = DAInf_BuildTail( inf );
	}

	// 構築用の一時情報を破棄
	DAInf_Delete( inf );

	return	ret;
}

// 構築
static	int		DADic_Construct( hDADic h, hDASet set, int boundary )
{
	int		ret;
	hDASort		sort;

	// 設定
	h->nkey = set->n;
	if ( ( ret = DADic_Boundary( h, boundary ) ) != DA_OK )
		return	ret;

	// 整列キー集合の作成
	if ( ( ret = DASort_Open( &sort, set ) ) != DA_OK )
		return	ret;

	// 辞書の構築
	ret = DADic_Edit( h, sort );

	// 整列キー集合の破棄
	DASort_Close( sort );

	return	ret;
}


// 検索


// 完全一致検索
static	void	*DADic_ExactMatch( hDADic h, const char *key )
{
	int		idx;
	char	*tail;

	idx = DADic_Base( h, IDX_ROOT ) ^ ( unsigned char )*key;
	while ( !DADic_Leaf( h, idx ) )
	{
		// 遷移確認
		if ( DADic_Check( h, idx ) != ( unsigned char )*key )
			return	NULL;
		idx = DADic_Base( h, idx ) ^ ( unsigned char )*++key;
	}

	// サフィックスの確認
	tail = DADic_Str( h, DADic_Pos( h, idx ) );
	if ( ( tail = DAStr_Match( tail, key ) ) == NULL )
		return	NULL;

	// レコードの開始位置
	return	tail + h->rec * DADic_Rec( h, idx );
}

// 辞書からの前方一致検索
static	void	*DADic_PrefixMatch( hDADic h, const char **s, int *idx )
{
	int		leaf, flag = 0;
	char	*tail = NULL;

	// 開始位置の調整
	if ( *idx == IDX_ROOT )
		--*s;

	while ( !DADic_Leaf( h, *idx ) )
	{
		// 終端文字による遷移を確認
		if ( *++*s != '\0' )
		{
			leaf = DADic_Base( h, *idx ) ^ '\0';
			if ( DADic_Leaf( h, leaf ) )
			{
				tail = DADic_Str( h, DADic_Pos( h, leaf ) );
				if ( ( flag = ( *tail++ == '\0' ) ) )
					tail += h->rec * DADic_Rec( h, leaf );
			}
		}

		// 遷移確認
		*idx = DADic_Base( h, *idx ) ^ ( unsigned char )**s;
		if ( !DADic_Leaf( h, *idx ) )
		{
			if ( DADic_Check( h, *idx ) != ( unsigned char )**s )
			{
				*idx = -1;
				return	flag ? tail : NULL;
			}
		}

		// プレフィックスを検出
		if ( flag )
			return	tail;
	}

	// サフィックスの確認
	tail = DADic_Str( h, DADic_Pos( h, *idx ) );
	tail = DAStr_Prefix( tail, s );
	if ( tail != NULL )
		tail += h->rec * DADic_Rec( h, *idx );
	*idx = -1;

	// レコードの開始位置
	return	tail;
}


// 辞書のヘッダ


// ヘッダ確認
static	int		DAHead_Check( hDAHead h )
{
	if ( memcmp( h->str, HEAD_STR, sizeof( h->str ) ) )
		return	DA_EFMT;

	return	DA_OK;
}

// ヘッダ読み込み
static	int		DAHead_Read( hDAHead h, FILE *fp )
{
	if ( fread( h, sizeof( *h ), 1, fp ) != 1 )
		return	DA_EIO;

	return	DAHead_Check( h );
}

// ヘッダ書き込み
static	int		DAHead_Write( hDAHead h, FILE *fp )
{
	if ( fwrite( h, sizeof( *h ), 1, fp ) != 1 )
		return	DA_EIO;

	return	DA_OK;
}


// 辞書の入出力


// ヘッダから辞書を復元
static	int		DADic_Restore( hDADic h, hDAHead head )
{
	int		ret;

	// 辞書情報
	h->nkey = head->nkey;
	DADic_SetMask( h, head->nbit );
	h->boundary = head->boundary;
	h->rec = head->rec;

	// 領域を確保
	if ( ( ret = DADic_ResizeBC( h, head->nbc ) ) != DA_OK ||
		 ( ret = DADic_ResizeTail( h, head->ntail ) ) != DA_OK )
		return	ret;

	return	DA_OK;
}

// ヘッダ生成
static	void	DADic_Head( hDADic h, hDAHead head )
{
	// 識別用情報
	memcpy( head->str, HEAD_STR, sizeof( head->str ) );
	head->version = DA_VERSION;

	// 辞書情報
	head->nbc = h->nbc;
	head->ntail = h->ntail;
	head->nkey = h->nkey;
	head->nbit = h->nbit;
	head->boundary = h->boundary;
	head->rec = h->rec;
}

// 辞書の入力
static	int		DADic_Read( hDADic h, FILE *fp )
{
	int		ret;
	DAHead	head;

	// ヘッダを入力
	if ( ( ret = DAHead_Read( &head, fp ) ) != DA_OK )
		return	ret;

	// ヘッダから辞書を復元
	if ( ( ret = DADic_Restore( h, &head ) ) != DA_OK )
		return	ret;

	// 本体を入力
	if ( fread( h->bc, sizeof( *h->bc ) * h->nbc, 1, fp ) != 1 )
		return	DA_EIO;
	else if ( h->ntail && fread( h->tail, h->ntail, 1, fp ) != 1 )
		return	DA_EIO;

	return	DA_OK;
}

// 辞書の読み込み
static	int		DADic_Load( hDADic h, const char *path )
{
	int		ret;
	FILE	*fp;

	if ( ( fp = fopen( path, "rb" ) ) == NULL )
		return	DA_EIO;

	// 辞書を入力
	ret = DADic_Read( h, fp );

	fclose( fp );

	return	ret;
}

// 辞書の出力
static	int		DADic_Write( hDADic h, FILE *fp )
{
	int		ret;
	DAHead	head;

	// ヘッダを出力
	DADic_Head( h, &head );
	if ( ( ret = DAHead_Write( &head, fp ) ) != DA_OK )
		return	ret;

	// 本体を出力
	if ( fwrite( h->bc, sizeof( *h->bc ) * h->nbc, 1, fp ) != 1 )
		return	DA_EIO;
	else if ( h->ntail && fwrite( h->tail, h->ntail, 1, fp ) != 1 )
		return	DA_EIO;

	return	DA_OK;
}


// 外部インタフェース


// 辞書の作成
int		DADic_Build( hDADic *h, hDASet set, int boundary )
{
	int		ret;

	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;
	*h = NULL;
	if ( set == NULL )
		return	DA_EARG;

	// 辞書を作成
	if ( ( *h = DADic_Create() ) == NULL )
		return	DA_EMEM;

	// 辞書の構築に失敗すれば破棄
	if ( ( ret = DADic_Construct( *h, set, boundary ) ) != DA_OK )
	{
		DADic_Delete( *h );
		*h = NULL;
	}

	return	ret;
}

// 辞書の読み込み
int		DADic_Open( hDADic *h, const char *path )
{
	int		ret;

	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;
	*h = NULL;
	if ( path == NULL )
		return	DA_EARG;

	// 辞書を作成
	if ( ( *h = DADic_Create() ) == NULL )
		return	DA_EHDL;

	// 辞書を読み込み
	if ( ( ret = DADic_Load( *h, path ) ) != DA_OK )
	{
		DADic_Delete( *h );
		*h = NULL;
	}

	return	ret;
}

// 辞書の破棄
int		DADic_Close( hDADic h )
{
	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;

	DADic_Delete( h );

	return	DA_OK;
}

// 辞書の保存
int		DADic_Save( hDADic h, const char *path )
{
	int		ret;
	FILE	*fp;

	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;
	else if ( path == NULL )
		return	DA_EARG;

	if ( ( fp = fopen( path, "wb" ) ) == NULL )
		return	DA_EIO;

	// 辞書を出力
	ret = DADic_Write( h, fp );

	fclose( fp );

	return	ret;
}

// 辞書情報の取得
int		DADic_Status( hDADic h, hDAStat st )
{
	// 引数を確認
	if ( h == NULL )
		return	DA_EHDL;
	else if ( st == NULL )
		return	DA_EARG;

	// 情報を取得
	st->nbc = h->nbc;
	st->ntail = h->ntail;
	st->size = h->nbc * sizeof( *h->bc ) + h->ntail;
	st->nkey = h->nkey;
	st->boundary = h->boundary;

	return	DA_OK;
}

// 辞書からの検索
void	*DADic_Search( hDADic h, const char *key )
{
	// 引数を確認
	if ( h == NULL || key == NULL )
		return	NULL;

	return	DADic_ExactMatch( h, key );
}

// 辞書からの前方一致検索
void	*DADic_Prefix( hDADic h, const char **s, int *idx )
{
	// 引数を確認
	if ( h == NULL || s == NULL || *s == NULL || idx == NULL )
		return	NULL;
	else if ( *idx < 0 || *idx >= h->nbc )
		return	NULL;

	return	DADic_PrefixMatch( h, s, idx );
}


/*
	エラーメッセージ
*/


// エラーメッセージ
const char	*DAErr_String( int err )
{
	switch ( err )
	{
	case	DA_OK:		// 成功
		return	"No error";
	case	DA_EHDL:	// 不正なハンドル
		return	"Invalid handle";
	case	DA_EMEM:	// メモリエラー
		return	"Memory allocation error";
	case	DA_EIO:		// 入出力エラー
		return	"I/O error";
	case	DA_EARG:	// 不正な引数
		return	"Invalid argument";
	case	DA_EMAX:	// サイズ上限
		return	"Too large";
	}

	return	"Unknown error";
}
